USE master
GO

-- Check if database exists and drop it
IF EXISTS(SELECT * FROM sys.databases where name = 'AsyncTrigger')
DROP DATABASE AsyncTrigger
GO

-- Create the database
CREATE DATABASE AsyncTrigger
GO

-- Enable Service Broker
ALTER DATABASE AsyncTrigger
SET ENABLE_BROKER
GO

USE AsyncTrigger
GO

-- Create our test table
CREATE TABLE Table_1(
[id] [int] PRIMARY KEY IDENTITY(1,1) NOT NULL,
[Col1] [int] NOT NULL,
[Col2] [int] NOT NULL,
[Sum] [int] NULL)
GO

-- This procedure will run asynchronously and execute our task
CREATE PROCEDURE usp_Processor
AS
BEGIN
SET NOCOUNT ON;

DECLARE @Handle UNIQUEIDENTIFIER;
DECLARE @MessageType SYSNAME;
DECLARE @Message XML;

RECEIVE TOP (1)
	@Handle = conversation_handle,
	@MessageType = message_type_name,
	@Message = message_body
	FROM [MyQ];

IF(@Handle IS NOT NULL AND @Message IS NOT NULL)
BEGIN

DECLARE @tmpT Table(id int, Col1 int, Col2 int, [Sum] int);
	INSERT INTO @tmpT SELECT
		Table1.Column1.value('@ID', 'INT') as id,
		Table1.Column1.value('@Col1', 'INT')as col1,
		Table1.Column1.value('@Col2', 'INT') as col2,
		Table1.Column1.value('@Sum', 'INT') as [sum]
		FROM @Message.nodes('/row') AS Table1(Column1)
	
	WAITFOR DELAY '000:00:05';
	UPDATE Table_1 set [Sum] = tmpT.Col1 + tmpT.Col2
		FROM @tmpT as tmpT Inner Join Table_1 as T
		ON tmpT.id = T.id;

END -- End If
END -- End Procedure usp_Processor

GO

-- Create Message Type
CREATE MESSAGE TYPE [MyMessage] VALIDATION = WELL_FORMED_XML;
GO

-- Create Contract
CREATE CONTRACT [MyContract] ([MyMessage] SENT BY ANY);
GO

-- Create Queue
CREATE QUEUE [MyQ] WITH ACTIVATION
(STATUS = ON, MAX_QUEUE_READERS = 1,
PROCEDURE_NAME = usp_Processor,
EXECUTE AS OWNER);
GO

-- Create Service
CREATE SERVICE [MyService] ON QUEUE [MyQ] ([MyContract]);
GO

-- This trigger will send our message to the queue
CREATE TRIGGER Table_1_Insert ON Table_1 AFTER INSERT
AS
BEGIN

SET NOCOUNT ON;

DECLARE @MessageBody XML;

SET @MessageBody = (SELECT ID, Col1, Col2, [Sum] FROM Inserted FOR XML RAW, TYPE);

DECLARE @Handle UNIQUEIDENTIFIER;

BEGIN DIALOG CONVERSATION @Handle
FROM SERVICE [MyService]
TO SERVICE 'MyService'
ON CONTRACT [MyContract]
WITH ENCRYPTION = OFF;

SEND ON CONVERSATION @Handle
MESSAGE TYPE [MyMessage](@MessageBody);

END
GO
 

-- TEST

-- 1. Wstawiamy liczby
INSERT INTO Table_1 (Col1, Col2) VALUES (3, 6)

-- 2. Natychmiastowo sprawdzamy: sum = NULL
SELECT * FROM Table_1

-- 3. Odczekujemy 10 sekund i sprawdzamy: sum = 9
SELECT * FROM Table_1